@@ -130,11 +130,11 @@ Rapid Spanning Tree BPDUs(RST BPDUs) format
 import binascii
 import struct
 from . import packet_base
-from ryu.lib.mac import haddr_to_bin
+from ryu.lib import addrconv
 
 
 # BPDU destination
-BRIDGE_GROUP_ADDRESS = '\x01\x80\xc2\x00\x00\x00'
+BRIDGE_GROUP_ADDRESS = '01:80:c2:00:00:00'
 
 
 PROTOCOL_IDENTIFIER = 0
@@ -145,6 +145,15 @@ TYPE_TOPOLOGY_CHANGE_BPDU = 128
 TYPE_RSTBPDU = 2
 DEFAULT_BRIDGE_PRIORITY = 32768
 DEFAULT_PORT_PRIORITY = 128
+PORT_PATH_COST_100KB = 200000000
+PORT_PATH_COST_1MB = 20000000
+PORT_PATH_COST_10MB = 2000000
+PORT_PATH_COST_100MB = 200000
+PORT_PATH_COST_1GB = 20000
+PORT_PATH_COST_10GB = 2000
+PORT_PATH_COST_100GB = 200
+PORT_PATH_COST_1TB = 20
+PORT_PATH_COST_10TB = 2
 DEFAULT_MAX_AGE = 20
 DEFAULT_HELLO_TIME = 2
 DEFAULT_FORWARD_DELAY = 15
@@ -158,6 +167,8 @@ class bpdu(packet_base.PacketBase):
     _PACK_LEN = struct.calcsize(_PACK_STR)
     _BPDU_TYPES = {}
 
+    _MIN_LEN = _PACK_LEN
+
     @staticmethod
     def register_bpdu_type(sub_cls):
         bpdu._BPDU_TYPES[sub_cls.BPDU_TYPE] = sub_cls
@@ -169,9 +180,9 @@ class bpdu(packet_base.PacketBase):
         assert hasattr(self, 'VERSION_ID')
         assert hasattr(self, 'BPDU_TYPE')
 
-        self.protocol_id = PROTOCOL_IDENTIFIER
-        self.version_id = self.VERSION_ID
-        self.bpdu_type = self.BPDU_TYPE
+        self._protocol_id = PROTOCOL_IDENTIFIER
+        self._version_id = self.VERSION_ID
+        self._bpdu_type = self.BPDU_TYPE
 
         if hasattr(self, 'check_parameters'):
             self.check_parameters()
@@ -194,8 +205,8 @@ class bpdu(packet_base.PacketBase):
             return buf, None, None
 
     def serialize(self, payload, prev):
-        return struct.pack(bpdu._PACK_STR, self.protocol_id,
-                           self.version_id, self.bpdu_type)
+        return struct.pack(bpdu._PACK_STR, self._protocol_id,
+                           self._version_id, self._bpdu_type)
 
 
 @bpdu.register_bpdu_type
@@ -205,7 +216,7 @@ class ConfigurationBPDUs(bpdu):
     An instance has the following attributes at least.
     Most of them are same to the on-wire counterparts but in host byte
     order.
-    __init__ takes the correspondig args in this order.
+    __init__ takes the corresponding args in this order.
 
     ========================== ===============================================
     Attribute                  Description
@@ -236,6 +247,11 @@ class ConfigurationBPDUs(bpdu):
     BPDU_TYPE = TYPE_CONFIG_BPDU
     _PACK_STR = '!BQIQHHHHH'
     PACK_LEN = struct.calcsize(_PACK_STR)
+    _TYPE = {
+        'ascii': [
+            'root_mac_address', "bridge_mac_address"
+        ]
+    }
 
     _BRIDGE_PRIORITY_STEP = 4096
     _PORT_PRIORITY_STEP = 16
@@ -243,10 +259,10 @@ class ConfigurationBPDUs(bpdu):
 
     def __init__(self, flags=0, root_priority=DEFAULT_BRIDGE_PRIORITY,
                  root_system_id_extension=0,
-                 root_mac_address=haddr_to_bin('00:00:00:00:00:00'),
+                 root_mac_address='00:00:00:00:00:00',
                  root_path_cost=0, bridge_priority=DEFAULT_BRIDGE_PRIORITY,
                  bridge_system_id_extension=0,
-                 bridge_mac_address=haddr_to_bin('00:00:00:00:00:00'),
+                 bridge_mac_address='00:00:00:00:00:00',
                  port_priority=DEFAULT_PORT_PRIORITY, port_number=0,
                  message_age=0, max_age=DEFAULT_MAX_AGE,
                  hello_time=DEFAULT_HELLO_TIME,
@@ -336,13 +352,15 @@ class ConfigurationBPDUs(bpdu):
         mac_addr_list = [format((mac_addr >> (8 * i)) & 0xff, '02x')
                          for i in range(0, 6)]
         mac_addr_list.reverse()
-        mac_address = binascii.a2b_hex(''.join(mac_addr_list))
+        mac_address_bin = binascii.a2b_hex(''.join(mac_addr_list))
+        mac_address = addrconv.mac.bin_to_text(mac_address_bin)
 
         return priority, system_id_extension, mac_address
 
     @staticmethod
     def encode_bridge_id(priority, system_id_extension, mac_address):
-        mac_addr = int(binascii.hexlify(mac_address), 16)
+        mac_addr = int(binascii.hexlify(addrconv.mac.text_to_bin(mac_address)),
+                       16)
         return ((priority + system_id_extension) << 48) + mac_addr
 
     @staticmethod
@@ -391,7 +409,7 @@ class RstBPDUs(ConfigurationBPDUs):
     An instance has the following attributes at least.
     Most of them are same to the on-wire counterparts but in host byte
     order.
-    __init__ takes the correspondig args in this order.
+    __init__ takes the corresponding args in this order.
 
     ========================== ===========================================
     Attribute                  Description
@@ -429,15 +447,15 @@ class RstBPDUs(ConfigurationBPDUs):
 
     def __init__(self, flags=0, root_priority=DEFAULT_BRIDGE_PRIORITY,
                  root_system_id_extension=0,
-                 root_mac_address=haddr_to_bin('00:00:00:00:00:00'),
+                 root_mac_address='00:00:00:00:00:00',
                  root_path_cost=0, bridge_priority=DEFAULT_BRIDGE_PRIORITY,
                  bridge_system_id_extension=0,
-                 bridge_mac_address=haddr_to_bin('00:00:00:00:00:00'),
+                 bridge_mac_address='00:00:00:00:00:00',
                  port_priority=DEFAULT_PORT_PRIORITY, port_number=0,
                  message_age=0, max_age=DEFAULT_MAX_AGE,
                  hello_time=DEFAULT_HELLO_TIME,
                  forward_delay=DEFAULT_FORWARD_DELAY):
-        self.version_1_length = VERSION_1_LENGTH
+        self._version_1_length = VERSION_1_LENGTH
 
         super(RstBPDUs, self).__init__(flags, root_priority,
                                        root_system_id_extension,
@@ -469,5 +487,5 @@ class RstBPDUs(ConfigurationBPDUs):
 
     def serialize(self, payload, prev):
         base = super(RstBPDUs, self).serialize(payload, prev)
-        sub = struct.pack(RstBPDUs._PACK_STR, self.version_1_length)
+        sub = struct.pack(RstBPDUs._PACK_STR, self._version_1_length)
         return base + sub
