@@ -14,9 +14,10 @@
 # limitations under the License.
 
 import abc
+from ryu.lib import stringify
 
 
-class PacketBase(object):
+class PacketBase(stringify.StringifyMixin):
     """A base class for a protocol (ethernet, ipv4, ...) header."""
     __metaclass__ = abc.ABCMeta
     _TYPES = {}
@@ -40,6 +41,9 @@ class PacketBase(object):
     def __init__(self):
         super(PacketBase, self).__init__()
 
+    def __len__(self):
+        return self._MIN_LEN
+
     @property
     def protocol_name(self):
         return self.__class__.__name__
@@ -52,21 +56,16 @@ class PacketBase(object):
         This method is used only when decoding a packet.
 
         Decode a protocol header at offset 0 in bytearray *buf*.
-        Returns the following two objects.
+        Returns the following three objects.
 
         * An object to describe the decoded header.
-          It should have the following attributes at least.
-
-          =========== ============
-          Attribute   Description
-          =========== ============
-          length      The number of the corresponding on-wire octets.
-          =========== ============
 
         * A packet_base.PacketBase subclass appropriate for the rest of
           the packet.  None when the rest of the packet should be considered
           as raw payload.
 
+        * The rest of packet.
+
         """
         pass
 
